@@ -3,6 +3,7 @@
 Modules:
     resource
     dataset
+    dataset_series
     catalog
     dataservice
     distribution
@@ -17,11 +18,14 @@ try:
 except PackageNotFoundError:  # pragma: no cover
     __version__ = "unknown"
 
+from .agent import Agent
 from .catalog import Catalog
 from .catalogrecord import CatalogRecord
 from .dataservice import DataService
 from .dataset import Dataset
+from .dataset_series import DatasetSeries
 from .distribution import Distribution
+from .document import Document
 from .exceptions import InvalidDateError, InvalidDateIntervalError
 from .location import Location
 from .periodoftime import PeriodOfTime
