@@ -2,7 +2,7 @@
 
 This module contains methods for mapping a catalogrecord object to rdf
 according to the
-`dcat-ap-no v.2 standard <https://doc.difi.no/review/dcat-ap-no/#klasse-katalog>`__
+`dcat-ap-no v.2 standard <https://data.norge.no/specification/dcat-ap-no/#klasse-katalogpost>`__
 
 Example:
     >>> from datacatalogtordf import CatalogRecord
@@ -17,9 +17,10 @@ Example:
 """
 from __future__ import annotations
 
-from typing import List, Optional
+from typing import Dict, List, Optional, Union
 
 from rdflib import Graph, Literal, Namespace, RDF, URIRef
+from skolemizer import Skolemizer
 
 from .periodoftime import Date
 from .resource import Resource
@@ -36,17 +37,8 @@ class CatalogRecord:
 
     Ref: https://www.w3.org/TR/vocab-dcat-2/#Class:Catalog_Record
 
-    Attributes:
-        identifier (URI): a URI uniquely identifying the catalog record
-        title (dict): A name given to the record. key is language code.
-        description (dict): A free-text account of the record. key is language code.
-        listing_date (Date): The date of listing
-        modification_date (Date): Most recent date on which the catalog entry \
-            was changed
-        primary_topic (Resource): The dcat:Resource (dataset or service) \
-            described in the record.
-        conforms_to (List[URI]): An established standard to which the described \
-            resource conforms.
+    Args:
+        identifier (URI): the identifier of the dataset-series.
     """
 
     __slots__ = (
@@ -62,20 +54,22 @@ class CatalogRecord:
 
     _g: Graph
     _identifier: URI
-    _title: dict
-    _description: dict
+    _title: Dict[str, str]
+    _description: Dict[str, str]
     _listing_date: Date
     _modification_date: Date
     _primary_topic: Resource
     _conforms_to: List[str]
 
-    def __init__(self) -> None:
+    def __init__(self, identifier: Optional[str] = None) -> None:
         """Inits catalogrecord object with default values."""
+        if identifier:
+            self.identifier = identifier
         self.conforms_to = []
 
     @property
     def identifier(self: CatalogRecord) -> str:
-        """Get/set for identifier."""
+        """URI: a URI uniquely identifying the catalog record."""
         return self._identifier
 
     @identifier.setter
@@ -83,26 +77,26 @@ class CatalogRecord:
         self._identifier = URI(identifier)
 
     @property
-    def title(self: CatalogRecord) -> dict:
-        """Title attribute."""
+    def title(self: CatalogRecord) -> Dict[str, str]:
+        """Dict[str, str]: A name given to the record. key is language code."""
         return self._title
 
     @title.setter
-    def title(self: CatalogRecord, title: dict) -> None:
+    def title(self: CatalogRecord, title: Dict[str, str]) -> None:
         self._title = title
 
     @property
-    def description(self: CatalogRecord) -> dict:
-        """Description attribute."""
+    def description(self: CatalogRecord) -> Dict[str, str]:
+        """Dict[str, str]: A free-text account of the record. key is language code."""
         return self._description
 
     @description.setter
-    def description(self: CatalogRecord, description: dict) -> None:
+    def description(self: CatalogRecord, description: Dict[str, str]) -> None:
         self._description = description
 
     @property
     def listing_date(self: CatalogRecord) -> str:
-        """Get/set for listing_date."""
+        """Date: The date of listing."""
         return self._listing_date
 
     @listing_date.setter
@@ -111,7 +105,7 @@ class CatalogRecord:
 
     @property
     def modification_date(self: CatalogRecord) -> str:
-        """Get/set for modification_date."""
+        """Date: Most recent date on which the catalog entry was changed."""
         return self._modification_date
 
     @modification_date.setter
@@ -120,7 +114,7 @@ class CatalogRecord:
 
     @property
     def primary_topic(self: CatalogRecord) -> Resource:
-        """Get/set for primary_topic."""
+        """Resource: The dcat:Resource (dataset or service) described in the record."""  # noqa: B950
         return self._primary_topic
 
     @primary_topic.setter
@@ -129,7 +123,7 @@ class CatalogRecord:
 
     @property
     def conforms_to(self: CatalogRecord) -> List[str]:
-        """Get/set for conforms_to."""
+        """List[URI]: An established standard to which the described resource conforms."""
         return self._conforms_to
 
     @conforms_to.setter
@@ -139,13 +133,16 @@ class CatalogRecord:
     # -
     def to_rdf(
         self: CatalogRecord, format: str = "turtle", encoding: Optional[str] = "utf-8"
-    ) -> bytes:
+    ) -> Union[bytes, str]:
         """Maps the catalogrecord to rdf."""
         return self._to_graph().serialize(format=format, encoding=encoding)
 
     # -
     def _to_graph(self: CatalogRecord) -> Graph:
 
+        if not getattr(self, "identifier", None):
+            self.identifier = Skolemizer.add_skolemization()
+
         # set up graph and namespaces:
         self._g = Graph()
         self._g.bind("dct", DCT)
@@ -208,6 +205,10 @@ class CatalogRecord:
 
     def _primary_topic_to_graph(self: CatalogRecord) -> None:
         if getattr(self, "primary_topic", None):
+
+            if not getattr(self.primary_topic, "identifier", None):
+                self.primary_topic.identifier = Skolemizer.add_skolemization()
+
             self._g.add(
                 (
                     URIRef(self.identifier),
