@@ -2,7 +2,7 @@
 
 This module contains methods for mapping a distribution object to rdf
 according to the
-`dcat-ap-no v.2 standard <https://doc.difi.no/review/dcat-ap-no/#klasse-distribusjon>`__
+`dcat-ap-no v.2 standard <https://data.norge.no/specification/dcat-ap-no/#klasse-distribusjon>`__
 
 Example:
     >>> from datacatalogtordf import Distribution
@@ -17,10 +17,11 @@ Example:
 from __future__ import annotations
 
 from decimal import Decimal
-from typing import List, Optional, TYPE_CHECKING
+from typing import Dict, List, Optional, TYPE_CHECKING, Union
 
 from rdflib import Graph, Literal, Namespace, RDF, URIRef
 from rdflib.namespace import DCTERMS
+from skolemizer import Skolemizer
 
 from .periodoftime import Date
 from .uri import URI
@@ -39,45 +40,8 @@ class Distribution:
 
     Ref: `dcat:Distribution <https://www.w3.org/TR/vocab-dcat-2/#Class:Distribution>`_
 
-    Attributes:
-        identifier (URI): A URI uniquely identifying the resource
-        title (dict): A dict with title in multiple languages
-        description (dict): A free-text account of the distribution.
-        release_date (Date): Date of formal issuance (e.g., publication) \
-            of the distribution.
-        modification_date (Date): Most recent date on which the distribution \
-            was changed, updated or modified.
-        license (URI): A link to legal document under which the distribution is \
-            made available.
-        access_rights (URI): A link to rights statement that concerns how the \
-            distribution is accessed.
-        rights (URI): A link to information about rights held in and over \
-            the distribution.
-        has_policy (URI): A link to an ODRL conformant policy expressing \
-            the rights associated with the distribution.
-        access_URL (URI): A URL of the resource that gives access to a \
-            distribution of the dataset. E.g. landing page, feed, SPARQL endpoint.
-        access_service (DataService): A data service that gives access to \
-            the distribution of the dataset
-        download_URL (URI): The URL of the downloadable file in a given format.\
-            E.g. CSV file or RDF file. The format is indicated by \
-            the distribution's dct:format and/or dcat:mediaType
-        byte_size (Decimal): 	The size of a distribution in bytes.
-        spatial_resolution (Decimal): 	The minimum spatial separation resolvable \
-            in a dataset distribution, measured in meters.
-        temporal_resolution (str): Minimum time period resolvable in the \
-            dataset distribution.
-        conforms_to (List[URI]): A list of links to established standards \
-            to which the distribution conforms.
-        media_types (List[URI]): A list of media types of the distribution \
-            as defined by IANA.
-        formats (List[URI]): A list of file formats of the distribution.
-        compression_format (URI): The compression format of the distribution \
-            in which the data is contained in a compressed form, e.g. \
-            to reduce the size of the downloadable file.
-        package_format (URI): The package format of the distribution in which \
-            one or more data files are grouped together, e.g. to enable a set \
-            of related files to be downloaded together.
+    Args:
+        identifier (URI): the identifier of the dataset-series.
     """
 
     __slots__ = (
@@ -95,7 +59,7 @@ class Distribution:
         "_access_service",
         "_download_URL",
         "_byte_size",
-        "_spatial_resolution",
+        "_spatial_resolution_in_meters",
         "_temporal_resolution",
         "_conforms_to",
         "_media_types",
@@ -106,8 +70,8 @@ class Distribution:
 
     _g: Graph
     _identifier: URI
-    _title: dict
-    _description: dict
+    _title: Dict[str, str]
+    _description: Dict[str, str]
     _release_date: Date
     _modification_date: Date
     _license: URI
@@ -118,23 +82,26 @@ class Distribution:
     _access_service: DataService
     _download_URL: URI
     _byte_size: Decimal
-    _spatial_resolution: Decimal
-    _temporal_resolution: str
+    _spatial_resolution_in_meters: List[Decimal]
+    _temporal_resolution: List[str]
     _conforms_to: List[str]
     _media_types: List[str]
     _formats: List[str]
     _compression_format: URI
     _package_format: URI
 
-    def __init__(self) -> None:
+    def __init__(self, identifier: Optional[str] = None) -> None:
         """Inits an object with default values."""
+        if identifier:
+            self.identifier = identifier
+
         self.conforms_to = []
         self.media_types = []
         self.formats = []
 
     @property
     def identifier(self: Distribution) -> str:
-        """Get/set for identifier."""
+        """URI: A URI uniquely identifying the resource."""
         return self._identifier
 
     @identifier.setter
@@ -142,26 +109,26 @@ class Distribution:
         self._identifier = URI(identifier)
 
     @property
-    def title(self: Distribution) -> dict:
-        """Get/set for title."""
+    def title(self: Distribution) -> Dict[str, str]:
+        """Dict[str]: A dict with title in multiple languages."""
         return self._title
 
     @title.setter
-    def title(self: Distribution, title: dict) -> None:
+    def title(self: Distribution, title: Dict[str, str]) -> None:
         self._title = title
 
     @property
-    def description(self: Distribution) -> dict:
-        """Description attribute."""
+    def description(self: Distribution) -> Dict[str, str]:
+        """Dict[str]: A free-text account of the distribution."""
         return self._description
 
     @description.setter
-    def description(self: Distribution, description: dict) -> None:
+    def description(self: Distribution, description: Dict[str, str]) -> None:
         self._description = description
 
     @property
     def release_date(self: Distribution) -> str:
-        """Get/set for release_date."""
+        """Date: Date of formal issuance (e.g., publication) of the distribution."""
         return self._release_date
 
     @release_date.setter
@@ -170,7 +137,7 @@ class Distribution:
 
     @property
     def modification_date(self: Distribution) -> str:
-        """Get/set for modification_date."""
+        """Date: Most recent date on which the distribution was changed, updated or modified."""
         return self._modification_date
 
     @modification_date.setter
@@ -179,7 +146,7 @@ class Distribution:
 
     @property
     def license(self: Distribution) -> str:
-        """Get/set for license."""
+        """URI: A link to legal document under which the distribution is made available."""
         return self._license
 
     @license.setter
@@ -188,7 +155,7 @@ class Distribution:
 
     @property
     def access_rights(self: Distribution) -> str:
-        """Get/set for access_rights."""
+        """URI: A link to rights statement that concerns how the distribution is accessed."""
         return self._access_rights
 
     @access_rights.setter
@@ -197,7 +164,7 @@ class Distribution:
 
     @property
     def rights(self: Distribution) -> str:
-        """Get/set for rights."""
+        """URI: A link to information about rights held in and over the distribution."""
         return self._rights
 
     @rights.setter
@@ -206,7 +173,7 @@ class Distribution:
 
     @property
     def has_policy(self: Distribution) -> str:
-        """Get/set for has_policy."""
+        """URI: A link to an ODRL conformant policy expressing the rights associated with the distribution."""
         return self._has_policy
 
     @has_policy.setter
@@ -215,7 +182,7 @@ class Distribution:
 
     @property
     def access_URL(self: Distribution) -> str:
-        """Get/set for access_URL."""
+        """URI: A URL of the resource that gives access to a distribution of the dataset. E.g. landing page, feed, SPARQL endpoint."""  # noqa: B950
         return self._access_URL
 
     @access_URL.setter
@@ -224,7 +191,7 @@ class Distribution:
 
     @property
     def access_service(self: Distribution) -> DataService:
-        """Get/set for access_service."""
+        """DataService: A data service that gives access to the distribution of the dataset."""  # noqa: B950
         return self._access_service
 
     @access_service.setter
@@ -233,7 +200,7 @@ class Distribution:
 
     @property
     def download_URL(self: Distribution) -> str:
-        """Get/set for download_URL."""
+        """URI: The URL of the downloadable file in a given format. E.g. CSV file or RDF file. The format is indicated by the distribution's dct:format and/or dcat:mediaType."""  # noqa: B950
         return self._download_URL
 
     @download_URL.setter
@@ -242,7 +209,7 @@ class Distribution:
 
     @property
     def byte_size(self: Distribution) -> Decimal:
-        """Get/set for byte_size."""
+        """Decimal: The size of a distribution in bytes."""
         return self._byte_size
 
     @byte_size.setter
@@ -250,26 +217,28 @@ class Distribution:
         self._byte_size = byte_size
 
     @property
-    def spatial_resolution(self: Distribution) -> Decimal:
-        """Get/set for spatial_resolution."""
-        return self._spatial_resolution
+    def spatial_resolution_in_meters(self: Distribution) -> List[Decimal]:
+        """List[Decimal]: A list of minimum spatial separation resolvables in a dataset distribution, measured in meters."""  # noqa: B950
+        return self._spatial_resolution_in_meters
 
-    @spatial_resolution.setter
-    def spatial_resolution(self: Distribution, spatial_resolution: Decimal) -> None:
-        self._spatial_resolution = spatial_resolution
+    @spatial_resolution_in_meters.setter
+    def spatial_resolution_in_meters(
+        self: Distribution, spatial_resolution_in_meters: List[Decimal]
+    ) -> None:
+        self._spatial_resolution_in_meters = spatial_resolution_in_meters
 
     @property
-    def temporal_resolution(self: Distribution) -> str:
-        """Get/set for temporal_resolution."""
+    def temporal_resolution(self: Distribution) -> List[str]:
+        """List[str]: A list of minimum time period resolvables in the dataset distribution."""
         return self._temporal_resolution
 
     @temporal_resolution.setter
-    def temporal_resolution(self: Distribution, temporal_resolution: str) -> None:
+    def temporal_resolution(self: Distribution, temporal_resolution: List[str]) -> None:
         self._temporal_resolution = temporal_resolution
 
     @property
     def conforms_to(self: Distribution) -> List[str]:
-        """Get/set for conforms_to."""
+        """List[URI]: A list of links to established standards to which the distribution conforms."""
         return self._conforms_to
 
     @conforms_to.setter
@@ -278,7 +247,7 @@ class Distribution:
 
     @property
     def media_types(self: Distribution) -> List[str]:
-        """Get/set for media_types."""
+        """List[URI]: A list of media types of the distribution as defined by IANA."""
         return self._media_types
 
     @media_types.setter
@@ -287,7 +256,7 @@ class Distribution:
 
     @property
     def formats(self: Distribution) -> List[str]:
-        """Get/set for formats."""
+        """List[URI]: A list of file formats of the distribution."""
         return self._formats
 
     @formats.setter
@@ -296,7 +265,7 @@ class Distribution:
 
     @property
     def compression_format(self: Distribution) -> str:
-        """Get/set for compression_format."""
+        """URI: Link to the compression format of the distribution in which the data is contained in a compressed form, e.g. to reduce the size of the downloadable file."""  # noqa: B950
         return self._compression_format
 
     @compression_format.setter
@@ -305,7 +274,7 @@ class Distribution:
 
     @property
     def package_format(self: Distribution) -> str:
-        """Get/set for package_format."""
+        """URI: Link to the package format of the distribution in which one or more data files are grouped together, e.g. to enable a set of related files to be downloaded together."""  # noqa: B950
         return self._package_format
 
     @package_format.setter
@@ -315,13 +284,16 @@ class Distribution:
     # -
     def to_rdf(
         self: Distribution, format: str = "turtle", encoding: Optional[str] = "utf-8"
-    ) -> bytes:
+    ) -> Union[bytes, str]:
         """Maps the distribution to rdf."""
         return self._to_graph().serialize(format=format, encoding=encoding)
 
     # -
     def _to_graph(self: Distribution) -> Graph:
 
+        if not getattr(self, "identifier", None):
+            self.identifier = Skolemizer.add_skolemization()
+
         # set up graph and namespaces:
         self._g = Graph()
         self._g.bind("dct", DCT)
@@ -342,7 +314,7 @@ class Distribution:
         self._access_service_to_graph()
         self._download_URL_to_graph()
         self._byte_size_to_graph()
-        self._spatial_resolution_to_graph()
+        self._spatial_resolution_in_meters_to_graph()
         self._temporal_resolution_to_graph()
         self._conforms_to_to_graph()
         self._media_types_to_graph()
@@ -422,6 +394,10 @@ class Distribution:
 
     def _access_service_to_graph(self: Distribution) -> None:
         if getattr(self, "access_service", None):
+
+            if not getattr(self.access_service, "identifier", None):
+                self.access_service.identifier = Skolemizer.add_skolemization()
+
             self._g.add(
                 (
                     URIRef(self.identifier),
@@ -446,25 +422,27 @@ class Distribution:
                 )
             )
 
-    def _spatial_resolution_to_graph(self: Distribution) -> None:
-        if getattr(self, "spatial_resolution", None):
-            self._g.add(
-                (
-                    URIRef(self.identifier),
-                    DCAT.spatialResolutionInMeters,
-                    Literal(self.spatial_resolution, datatype=XSD.decimal),
+    def _spatial_resolution_in_meters_to_graph(self: Distribution) -> None:
+        if getattr(self, "spatial_resolution_in_meters", None):
+            for resolution in self.spatial_resolution_in_meters:
+                self._g.add(
+                    (
+                        URIRef(self.identifier),
+                        DCAT.spatialResolutionInMeters,
+                        Literal(resolution, datatype=XSD.decimal),
+                    )
                 )
-            )
 
     def _temporal_resolution_to_graph(self: Distribution) -> None:
         if getattr(self, "temporal_resolution", None):
-            self._g.add(
-                (
-                    URIRef(self.identifier),
-                    DCAT.temporalResolution,
-                    Literal(self.temporal_resolution, datatype=XSD.duration),
+            for temporal_resolution in self.temporal_resolution:
+                self._g.add(
+                    (
+                        URIRef(self.identifier),
+                        DCAT.temporalResolution,
+                        Literal(temporal_resolution, datatype=XSD.duration),
+                    )
                 )
-            )
 
     def _conforms_to_to_graph(self: Distribution) -> None:
         if getattr(self, "conforms_to", None):
