@@ -2,7 +2,7 @@
 
 This module contains methods for mapping a location object to rdf
 according to the
-`dcat-ap-no v.2 standard <https://doc.difi.no/review/dcat-ap-no/#klasse-distribusjon>`__
+`dcat-ap-no v.2 standard <https://data.norge.no/specification/dcat-ap-no/#klasse-lokasjon>`__
 
 Example:
     >>> from datacatalogtordf import Location
@@ -16,9 +16,10 @@ Example:
 """
 from __future__ import annotations
 
-from typing import Optional
+from typing import Optional, Union
 
-from rdflib import BNode, Graph, Literal, Namespace, RDF, URIRef
+from rdflib import Graph, Literal, Namespace, RDF, URIRef
+from skolemizer import Skolemizer
 
 from .uri import URI
 
@@ -33,12 +34,6 @@ class Location:
     """A class representing a dcat:Location.
 
     Ref: `dcat:Location <https://www.w3.org/TR/vocab-dcat-2/#Class:Location>`_
-
-    Attributes:
-        identifier (URI): an URI uniquely identifying the resource
-        geometry (str): Associates any resource with the corresponding geometry.
-        bounding_box (str): The geographic bounding box of a resource.
-        centroid (str): The geographic center (centroid) of a resource.
     """
 
     slots = ("_identifier", "_geometry", "_bounding_box", "_centroid")
@@ -49,8 +44,11 @@ class Location:
     _centroid: str
     _ref: URIRef
 
-    def __init__(self) -> None:
+    def __init__(self, identifier: Optional[str] = None) -> None:
         """Inits an object with default values."""
+        if identifier:
+            self.identifier = identifier
+
         # set up graph and namespaces:
         self._g = Graph()
         self._g.bind("dct", DCT)
@@ -60,7 +58,7 @@ class Location:
 
     @property
     def identifier(self: Location) -> str:
-        """Get/set for identifier."""
+        """URI: an URI uniquely identifying the resource."""
         return self._identifier
 
     @identifier.setter
@@ -69,7 +67,7 @@ class Location:
 
     @property
     def geometry(self: Location) -> str:
-        """Get/set for geometry."""
+        """str: Associates any resource with the corresponding geometry."""
         return self._geometry
 
     @geometry.setter
@@ -78,7 +76,7 @@ class Location:
 
     @property
     def bounding_box(self: Location) -> str:
-        """Get/set for bounding_box."""
+        """str: The geographic bounding box of a resource."""
         return self._bounding_box
 
     @bounding_box.setter
@@ -87,7 +85,7 @@ class Location:
 
     @property
     def centroid(self: Location) -> str:
-        """Get/set for centroid."""
+        """str: The geographic center (centroid) of a resource."""
         return self._centroid
 
     @centroid.setter
@@ -97,7 +95,7 @@ class Location:
     # -
     def to_rdf(
         self: Location, format: str = "turtle", encoding: Optional[str] = "utf-8"
-    ) -> bytes:
+    ) -> Union[bytes, str]:
         """Maps the location to rdf.
 
         Args:
@@ -112,10 +110,10 @@ class Location:
     # -
     def _to_graph(self: Location) -> Graph:
 
-        if getattr(self, "identifier", None):
-            self._ref = URIRef(self.identifier)
-        else:
-            self._ref = BNode()
+        if not getattr(self, "identifier", None):
+            self.identifier = Skolemizer.add_skolemization()
+
+        self._ref = URIRef(self.identifier)
         self._g.add((self._ref, RDF.type, DCT.Location))
 
         self._geometry_to_graph()
